#include "StdAfx.h"
#include "resource.h"
#include "LeoHelpers.h"
#include "JP2RawConfig.h"
#include "dcraw.h"
#include "Win32IOWrapper.h"
#include "LeoThemeHelper.h"

NJP2RawConfig::CDropTarget::CDropTarget()
: m_bDragEnabled(true)
, m_hwnd(NULL)
{
}

NJP2RawConfig::CDropTarget::~CDropTarget()
{
	Revoke();
}

void NJP2RawConfig::CDropTarget::SetDragEnabled(bool bEnabled)
{
	m_bDragEnabled = bEnabled;
}

void NJP2RawConfig::CDropTarget::AddDropWindow(HWND hWnd)
{
	m_vecDropWindows.push_back( hWnd );
}

void NJP2RawConfig::CDropTarget::Register(HWND hwnd)
{
	Revoke();

	if (NULL != hwnd)
	{
		m_hwnd = hwnd;
		RegisterDragDrop(hwnd, this);
	}
}

void NJP2RawConfig::CDropTarget::Revoke()
{
	if (NULL != m_hwnd)
	{
		RevokeDragDrop(m_hwnd);
		m_hwnd = NULL;
	}
}

std::vector< std::basic_string<TCHAR> > &NJP2RawConfig::CDropTarget::GetFilenames()
{
	return m_vecFilenames;
}

HWND NJP2RawConfig::CDropTarget::GetDropWindowFromScreenPoint(POINT &ptScreen)
{
	HWND hWndDrop = NULL;

	if (m_bDragEnabled)
	{
		RECT rect;

		for (std::vector< HWND >::iterator pHwnd = m_vecDropWindows.begin(); pHwnd != m_vecDropWindows.end(); ++pHwnd)
		{
			DWORD dwStyle = GetWindowLong(*pHwnd, GWL_STYLE);

			if ( (dwStyle & WS_VISIBLE)
			&&	!(dwStyle & WS_DISABLED)
			&&	GetWindowRect(*pHwnd, &rect)
			&&	PtInRect(&rect, ptScreen))
			{
				bool bAllParentsSuitable = true;

				HWND hWnd = *pHwnd;

				while(dwStyle & WS_CHILD)
				{
					hWnd = GetParent(hWnd);
					dwStyle = GetWindowLong(hWnd, GWL_STYLE);

					if (!(dwStyle & WS_VISIBLE) || (dwStyle & WS_DISABLED))
					{
						bAllParentsSuitable = false;
						break;
					}
				}

				if (bAllParentsSuitable)
				{
					hWndDrop = *pHwnd;
					break;
				}
			}
		}
	}

	return(hWndDrop);
}

// IUnknown
HRESULT STDMETHODCALLTYPE NJP2RawConfig::CDropTarget::QueryInterface(REFIID riid, void __RPC_FAR *__RPC_FAR *ppvObject)
{
	if (IID_IUnknown == riid)
	{
		*ppvObject = static_cast<IUnknown *>(this);
		return S_OK;
	}
	else if (IID_IDropTarget == riid)
	{
		*ppvObject = static_cast<IDropTarget *>(this);
		return S_OK;
	}
	else
	{
		*ppvObject = NULL;
		return E_NOINTERFACE;
	}
}

ULONG STDMETHODCALLTYPE NJP2RawConfig::CDropTarget::AddRef(void)
{
	return 1;
}

ULONG STDMETHODCALLTYPE NJP2RawConfig::CDropTarget::Release(void)
{
	return 1;
}

// IDropTarget
HRESULT STDMETHODCALLTYPE NJP2RawConfig::CDropTarget::DragEnter(IDataObject *pDataObj, DWORD grfKeyState, POINTL pt, DWORD *pdwEffect)
{
	POINT pt2 = { pt.x, pt.y };
	*pdwEffect = (NULL != GetDropWindowFromScreenPoint(pt2) ? DROPEFFECT_COPY : DROPEFFECT_NONE);
	return S_OK;
}

HRESULT STDMETHODCALLTYPE NJP2RawConfig::CDropTarget::DragOver(DWORD grfKeyState, POINTL pt, DWORD *pdwEffect)
{
	POINT pt2 = { pt.x, pt.y };
	*pdwEffect = (NULL != GetDropWindowFromScreenPoint(pt2) ? DROPEFFECT_COPY : DROPEFFECT_NONE);
	return S_OK;
}

HRESULT STDMETHODCALLTYPE NJP2RawConfig::CDropTarget::DragLeave(void)
{
	return S_OK;
}

HRESULT STDMETHODCALLTYPE NJP2RawConfig::CDropTarget::Drop(IDataObject *pDataObj, DWORD grfKeyState, POINTL pt, DWORD *pdwEffect)
{
	POINT pt2 = { pt.x, pt.y };
	HWND hWndDrop = GetDropWindowFromScreenPoint(pt2);
	if (NULL != hWndDrop)
	{
		FORMATETC format;
		format.cfFormat = CF_HDROP;
		format.ptd = NULL;
		format.dwAspect = DVASPECT_CONTENT;
		format.lindex = -1;
		format.tymed = TYMED_HGLOBAL;

		STGMEDIUM medium;
		ZeroMemory(&medium, sizeof(medium));

		if (SUCCEEDED(pDataObj->GetData(&format, &medium)))
		{
			if (medium.tymed == TYMED_HGLOBAL)
			{
				HDROP hDrop = reinterpret_cast<HDROP>(GlobalLock(medium.hGlobal));
				if (NULL != hDrop)
				{
					UINT uicDroppedFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);

					for (UINT uiFileIdx = 0; uiFileIdx < uicDroppedFiles; uiFileIdx++)
					{
						UINT uiccBuffer = DragQueryFile(hDrop, uiFileIdx, NULL, 0);
						if (0 < uiccBuffer)
						{
							uiccBuffer++; // Increment for null terminator.
							TCHAR *szBuffer = new TCHAR[uiccBuffer + 1]; // Add space for a second null because I don't trust the API.
							szBuffer[uiccBuffer] = _T('\0');			// Make sure that second null is there.

							if (0 < DragQueryFile(hDrop, uiFileIdx, szBuffer, uiccBuffer))
							{
								m_vecFilenames.push_back( szBuffer );
							}

							delete [] szBuffer;
						}
					}

					GlobalUnlock(medium.hGlobal);
				}
			}

			ReleaseStgMedium(&medium);
		}

		if (!m_vecFilenames.empty())
		{
			// Note: This is a non-standard use of the WM_DROPFILES message that m_hwnd must understand.
			PostMessage(m_hwnd, WM_DROPFILES, 0, reinterpret_cast<LPARAM>(hWndDrop));
		}
	}

	return S_OK;
}

BOOL CALLBACK NJP2RawConfig::configDlgProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	BOOL bResult = FALSE;
	bool bDeletePCD = false;
	bool bUpdateControls = false;
	bool bLoadSettings = false;
	bool bLoadDefaultSettings = false;
	bool bSaveSettings = false;
	bool bEnableDisable = false;

	NJR_ConfigData *pcd = reinterpret_cast<NJR_ConfigData *>(GetWindowLongPtr(hwndDlg, GWLP_USERDATA));

    switch (message)
    {
	case WM_INITDIALOG:
		configInitDialog(bResult, bDeletePCD, bUpdateControls, bLoadSettings, bLoadDefaultSettings, bSaveSettings, bEnableDisable, pcd, hwndDlg, message, wParam, lParam);
		break;
	case WM_NOTIFY:
		configNotify(    bResult, bDeletePCD, bUpdateControls, bLoadSettings, bLoadDefaultSettings, bSaveSettings, bEnableDisable, pcd, hwndDlg, message, wParam, lParam);
		break;
	case WM_VSCROLL:
		configVScroll(   bResult, bDeletePCD, bUpdateControls, bLoadSettings, bLoadDefaultSettings, bSaveSettings, bEnableDisable, pcd, hwndDlg, message, wParam, lParam);
		break;
	case WM_COMMAND:
		configCommand(   bResult, bDeletePCD, bUpdateControls, bLoadSettings, bLoadDefaultSettings, bSaveSettings, bEnableDisable, pcd, hwndDlg, message, wParam, lParam);
		break;
	case WM_DROPFILES:
		configDropFiles( bResult, bDeletePCD, bUpdateControls, bLoadSettings, bLoadDefaultSettings, bSaveSettings, bEnableDisable, pcd, hwndDlg, message, wParam, lParam);
		break;
	//case WM_SYSCOLORCHANGE:
	//	bResult = FALSE; // Pass on to standard handling.
	//	break;
	default:
		break;
	}

	if (NULL != pcd && (bLoadSettings || bLoadDefaultSettings))
	{
		configLoadSettings(pcd, bLoadDefaultSettings, message == WM_INITDIALOG);
		bEnableDisable = true;
	}

	if (NULL != pcd && bSaveSettings)
	{
		configSaveSettings(pcd);
	}

	if (NULL != pcd && bEnableDisable) // Must be after settings load
	{
		configEnableDisable(pcd);
	}

	if (bDeletePCD && NULL != pcd)
	{
		SetWindowLongPtr(pcd->hWndMainDlg,      GWLP_USERDATA, NULL);
		SetWindowLongPtr(pcd->hWndFormats,      GWLP_USERDATA, NULL);
		SetWindowLongPtr(pcd->hWndDCRawThumbs,  GWLP_USERDATA, NULL);
		SetWindowLongPtr(pcd->hWndDCRawViewers, GWLP_USERDATA, NULL);
		delete pcd;
	}

	return(bResult);
}

BOOL CALLBACK NJP2RawConfig::childProxyDlgProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
	case WM_INITDIALOG:
		{
			NJR_ConfigData *pcd = reinterpret_cast<NJR_ConfigData *>(lParam);
			SetWindowLongPtr(hwndDlg, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(pcd));
		}
		break;
	default:
		return (NJP2RawConfig::configDlgProc(hwndDlg, message, wParam, lParam));
		break;
	}

	return(FALSE);
}

BOOL CALLBACK NJP2RawConfig::tongueDlgProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	BOOL bResult = FALSE;

	UINT uiImage = 0;

	NJR_ConfigData *pcd = reinterpret_cast<NJR_ConfigData *>(GetWindowLongPtr(hwndDlg, GWLP_USERDATA));

	switch (message)
    {
	case WM_INITDIALOG:
		{
			pcd = reinterpret_cast<NJR_ConfigData *>(lParam);
			SetWindowLongPtr(hwndDlg, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(pcd));

			HWND hWndSlider = GetDlgItem(hwndDlg, IDC_SLIDER_TONGUE);
			if (NULL != hWndSlider)
			{
				SendMessage(hWndSlider, TBM_SETRANGE, TRUE, MAKELONG(0, 3));
			}
		}
		break;
	case WM_VSCROLL:
		{
			HWND hWndSlider = GetDlgItem(hwndDlg, IDC_SLIDER_TONGUE);
			if (NULL != hWndSlider
			&&	lParam == reinterpret_cast<LPARAM>(hWndSlider)
			&&	SB_ENDSCROLL != LOWORD(wParam))
			{
				//switch(HIWORD(wParam))
				switch(SendMessage(hWndSlider, TBM_GETPOS, 0, 0))
				{
				default:
				case(0): uiImage = IDR_JP21; break;
				case(1): uiImage = IDR_JP23; break;
				case(2): uiImage = IDR_JP22; break;
				case(3): uiImage = IDR_JP24; break;
				}
				bResult = TRUE;
			}
		}
		break;
	default:
		break;
	}

	if (0 != uiImage && NULL != pcd)
	{
		HWND hWndBitmap = GetDlgItem(hwndDlg, IDC_STATIC_TONGUE);

		if (NULL != hWndBitmap)
		{
			CxImage xi;
			if (xi.LoadResource(FindResource(pcd->hInstance, MAKEINTRESOURCE(uiImage), "JP2"), CXIMAGE_FORMAT_JP2, pcd->hInstance))
			{
				HBITMAP hbm = NLeoHelpers::MakeBitmapWithAlpha(xi, true);

				if (NULL != hbm)
				{
					HBITMAP hbmOld = reinterpret_cast<HBITMAP>(SendMessage(hWndBitmap, STM_SETIMAGE, IMAGE_BITMAP, reinterpret_cast<LPARAM>(hbm)));
					if (NULL != hbmOld)
					{
						// The static control made a copy of the previous image which must be deleted when we set a new one.
						DeleteObject(hbmOld);
					}
					// Delete our original copy of the new image.
					DeleteObject(hbm);
				}
			}
		}
	}

	return(bResult);
}

LRESULT CALLBACK NJP2RawConfig::editFloatFilterSubProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// Ignore all but numbers and decimal
	if (uMsg == WM_CHAR)
	{
		if (!isdigit(wParam) && wParam!='\b')
		{
			if (wParam=='.')
			{
				TCHAR tchBuf[256];
				GetWindowText(hWnd,tchBuf,256);
				if (strchr(tchBuf,_T('.')))
				{
					MessageBeep(-1);
					return 0;
				}
			}
			else
			{
				MessageBeep(-1);
				return 0;
			}
		}
	}

	return CallWindowProc(reinterpret_cast<WNDPROC>(GetWindowLongPtr(hWnd,GWLP_USERDATA)),hWnd,uMsg,wParam,lParam);
}

void NJP2RawConfig::configInitDialog(BOOL &bResult, bool &bDeletePCD, bool &bUpdateControls, bool &bLoadSettings, bool &bLoadDefaultSettings, bool &bSaveSettings, bool &bEnableDisable, NJR_ConfigData *&pcd, HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hWndTab = GetDlgItem(hwndDlg, IDC_TAB);
	if (NULL != hWndTab)
	{
		pcd = new NJR_ConfigData(*reinterpret_cast<NJR_ConfigData *>(lParam));
		SetWindowLongPtr(hwndDlg, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(pcd));

		pcd->hWndMainDlg      = hwndDlg;
		pcd->hWndFormats      = CreateDialogParam(pcd->hInstance, MAKEINTRESOURCE(IDD_FORMATS),    hwndDlg, NJP2RawConfig::childProxyDlgProc, reinterpret_cast<LPARAM>(pcd));
		pcd->hWndDCRawThumbs  = CreateDialogParam(pcd->hInstance, MAKEINTRESOURCE(IDD_DCRAW),      hwndDlg, NJP2RawConfig::childProxyDlgProc, reinterpret_cast<LPARAM>(pcd));
		pcd->hWndDCRawViewers = CreateDialogParam(pcd->hInstance, MAKEINTRESOURCE(IDD_DCRAW),      hwndDlg, NJP2RawConfig::childProxyDlgProc, reinterpret_cast<LPARAM>(pcd));
		pcd->hWndWithTongue   = CreateDialogParam(pcd->hInstance, MAKEINTRESOURCE(IDD_WITHTONGUE), hwndDlg, NJP2RawConfig::tongueDlgProc,     reinterpret_cast<LPARAM>(pcd));

		// This would be great if the stupid tab control didn't mysteriously block drag & drop to anything on top of it. Using IDropTarget instead.
		//HWND hWndListCamsThumbs  = GetDlgItem(pcd->hWndDCRawThumbs,  IDC_LIST_CAMERAS);
		//HWND hWndListCamsViewers = GetDlgItem(pcd->hWndDCRawViewers, IDC_LIST_CAMERAS);
		//if (NULL != hWndListCamsThumbs)  { DragAcceptFiles(hWndListCamsThumbs,  TRUE); }
		//if (NULL != hWndListCamsViewers) { DragAcceptFiles(hWndListCamsViewers, TRUE); }

		TCITEM tabItem;
		ZeroMemory(&tabItem, sizeof(tabItem));
		tabItem.mask = TCIF_PARAM|TCIF_TEXT;

		tabItem.pszText = "File Formats";
		tabItem.lParam = reinterpret_cast<LPARAM>(pcd->hWndFormats);
		SendMessage(hWndTab, TCM_INSERTITEM, 0, reinterpret_cast<LPARAM>(&tabItem));

		tabItem.pszText = "Raw Thumbnails";
		tabItem.lParam = reinterpret_cast<LPARAM>(pcd->hWndDCRawThumbs);
		SendMessage(hWndTab, TCM_INSERTITEM, 1, reinterpret_cast<LPARAM>(&tabItem));

		tabItem.pszText = "Raw Viewers";
		tabItem.lParam = reinterpret_cast<LPARAM>(pcd->hWndDCRawViewers);
		SendMessage(hWndTab, TCM_INSERTITEM, 2, reinterpret_cast<LPARAM>(&tabItem));

		RECT rectTabDisplay;
		GetWindowRect(hWndTab, &rectTabDisplay);
		NLeoHelpers::ScreenToClientRect(hwndDlg, &rectTabDisplay);
		TabCtrl_AdjustRect(hWndTab, FALSE, &rectTabDisplay);

		SetWindowPos(pcd->hWndFormats,      hWndTab, rectTabDisplay.left, rectTabDisplay.top, rectTabDisplay.right - rectTabDisplay.left, rectTabDisplay.bottom - rectTabDisplay.top, SWP_NOACTIVATE|SWP_NOOWNERZORDER);
		SetWindowPos(pcd->hWndDCRawThumbs,  hWndTab, rectTabDisplay.left, rectTabDisplay.top, rectTabDisplay.right - rectTabDisplay.left, rectTabDisplay.bottom - rectTabDisplay.top, SWP_NOACTIVATE|SWP_NOOWNERZORDER);
		SetWindowPos(pcd->hWndDCRawViewers, hWndTab, rectTabDisplay.left, rectTabDisplay.top, rectTabDisplay.right - rectTabDisplay.left, rectTabDisplay.bottom - rectTabDisplay.top, SWP_NOACTIVATE|SWP_NOOWNERZORDER);
		SetWindowPos(pcd->hWndWithTongue,   hWndTab, rectTabDisplay.left, rectTabDisplay.top, rectTabDisplay.right - rectTabDisplay.left, rectTabDisplay.bottom - rectTabDisplay.top, SWP_NOACTIVATE|SWP_NOOWNERZORDER);

		CLeoThemeHelper themeHelper;
		// The tab control doesn't resize and isn't too large for the stupid theme texture
		// so we can enable the texture and then forget about both it and the Theme Helper.
		themeHelper.EnableTabThemeDialogTexture(pcd->hWndFormats);
		themeHelper.EnableTabThemeDialogTexture(pcd->hWndDCRawThumbs);
		themeHelper.EnableTabThemeDialogTexture(pcd->hWndDCRawViewers);
		themeHelper.EnableTabThemeDialogTexture(pcd->hWndWithTongue);

		HWND hwndsFloatEditDlgs[] = { pcd->hWndDCRawThumbs, pcd->hWndDCRawViewers, NULL };
		int iFloatEditCtrlIds[] = { IDC_EDIT_BILAT_DOMAIN,
									IDC_EDIT_BILAT_RANGE,
									IDC_EDIT_USERMUL_1,
									IDC_EDIT_USERMUL_2,
									IDC_EDIT_USERMUL_3,
									IDC_EDIT_USERMUL_4,
									IDC_EDIT_BLACKPOINT,
									IDC_EDIT_BRIGHTNESS,
									0 };

		for (HWND *phwnd = hwndsFloatEditDlgs; *phwnd != NULL; phwnd++)
		{
			for (int *piEditCtrlId = iFloatEditCtrlIds; *piEditCtrlId != 0; piEditCtrlId++)
			{
				HWND hWndEdit = GetDlgItem(*phwnd, *piEditCtrlId);
				if (NULL != hWndEdit)
				{
					SetWindowLongPtr(hWndEdit, GWLP_USERDATA, SetWindowLongPtr(hWndEdit, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(editFloatFilterSubProc)));
				}
			}
		}

		TabCtrl_SetCurSel(hWndTab, 0);
		ShowWindow(pcd->hWndFormats, SW_SHOW);

		HWND hWndCameraListThumbs = GetDlgItem(pcd->hWndDCRawThumbs, IDC_LIST_CAMERAS);
		HWND hWndCameraListViewers = GetDlgItem(pcd->hWndDCRawViewers, IDC_LIST_CAMERAS);
		if (NULL != hWndCameraListThumbs && NULL != hWndCameraListViewers)
		{
			pcd->dropTarget.AddDropWindow(hWndCameraListThumbs);
			pcd->dropTarget.AddDropWindow(hWndCameraListViewers);
		}
		pcd->dropTarget.Register(pcd->hWndMainDlg);

		SendMessage(GetDlgItem(pcd->hWndFormats, IDC_RICHEDIT_CREDITS), EM_SETTARGETDEVICE, NULL, 0);
		SendMessage(GetDlgItem(pcd->hWndFormats, IDC_RICHEDIT_CREDITS), EM_AUTOURLDETECT, TRUE, 0);
		SendMessage(GetDlgItem(pcd->hWndFormats, IDC_RICHEDIT_CREDITS), EM_SETEVENTMASK, 0, SendMessage(GetDlgItem(pcd->hWndFormats, IDC_RICHEDIT_CREDITS), EM_GETEVENTMASK, 0, 0) | ENM_LINK);

		HRSRC hResCredits = FindResource(pcd->hInstance, MAKEINTRESOURCE(IDR_RTF_CREDITS), "RTF");
		HGLOBAL hGlobCredits = LoadResource(pcd->hInstance, hResCredits);
		char *szCredits = reinterpret_cast<char *>(LockResource(hGlobCredits));

		SetDlgItemText(pcd->hWndFormats, IDC_RICHEDIT_CREDITS, szCredits);

		NLeoHelpers::CenterWindow(pcd->hWndMainDlg, GetParent(pcd->hWndMainDlg));
		NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE); //FALSE); <-- Leave it enabled all the time. Too much hassle otherwise.

		bLoadSettings = true;
		bResult = TRUE;
	}
}

void NJP2RawConfig::configNotify(BOOL &bResult, bool &bDeletePCD, bool &bUpdateControls, bool &bLoadSettings, bool &bLoadDefaultSettings, bool &bSaveSettings, bool &bEnableDisable, NJR_ConfigData *pcd, HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (NULL != pcd)
	{
		NMHDR *pHdr = reinterpret_cast<NMHDR *>(lParam);
		switch(pHdr->code)
		{
		default:
			break;
		case(TCN_SELCHANGE):
			{
				TCITEM tabItem;
				ZeroMemory(&tabItem, sizeof(tabItem));
				tabItem.mask = TCIF_PARAM;
				if (TabCtrl_GetItem(pHdr->hwndFrom, TabCtrl_GetCurSel(pHdr->hwndFrom), &tabItem))
				{
					ShowWindow(pcd->hWndFormats,      tabItem.lParam == reinterpret_cast<LPARAM>(pcd->hWndFormats)      ? SW_SHOW : SW_HIDE);
					ShowWindow(pcd->hWndDCRawThumbs,  tabItem.lParam == reinterpret_cast<LPARAM>(pcd->hWndDCRawThumbs)  ? SW_SHOW : SW_HIDE);
					ShowWindow(pcd->hWndDCRawViewers, tabItem.lParam == reinterpret_cast<LPARAM>(pcd->hWndDCRawViewers) ? SW_SHOW : SW_HIDE);
					ShowWindow(pcd->hWndWithTongue,   tabItem.lParam == reinterpret_cast<LPARAM>(pcd->hWndWithTongue)   ? SW_SHOW : SW_HIDE);

					HWND hWndFocus = GetFocus();

					if (IsChild(pcd->hWndFormats,      hWndFocus)
					||	IsChild(pcd->hWndDCRawThumbs,  hWndFocus)
					||	IsChild(pcd->hWndDCRawViewers, hWndFocus)
					||	IsChild(pcd->hWndWithTongue,   hWndFocus))
					{
						SetFocus(reinterpret_cast<HWND>(tabItem.lParam));
					}

					if      (tabItem.lParam == reinterpret_cast<LPARAM>(pcd->hWndFormats))      { pcd->strActivePage = "Formats";    }
					else if (tabItem.lParam == reinterpret_cast<LPARAM>(pcd->hWndDCRawThumbs))  { pcd->strActivePage = "RawThumbs";  }
					else if (tabItem.lParam == reinterpret_cast<LPARAM>(pcd->hWndDCRawViewers)) { pcd->strActivePage = "RawViewers"; }
					// Don't remember the Tongue.
				}
			}
			break;
		case(EN_LINK):
			{
				if (wParam == IDC_RICHEDIT_CREDITS)
				{
					ENLINK *pEnLink = reinterpret_cast<ENLINK *>(lParam);
					if (pEnLink->msg == WM_LBUTTONDOWN)
					{
						char szBuffer[1024]; // Warning: Fixed size buffer. We assume none of the URLs in our credits text are longer.

						TEXTRANGE tr;
						tr.chrg = pEnLink->chrg;
						tr.lpstrText = szBuffer;
						SendMessage(GetDlgItem(pcd->hWndFormats, IDC_RICHEDIT_CREDITS),  EM_GETTEXTRANGE, 0, reinterpret_cast<LPARAM>(&tr));

						ShellExecute(NULL, _T("open"), szBuffer, NULL, NULL, SW_SHOW);
					}
				}
			}
			break;
		}
	}
}

void NJP2RawConfig::configVScroll(BOOL &bResult, bool &bDeletePCD, bool &bUpdateControls, bool &bLoadSettings, bool &bLoadDefaultSettings, bool &bSaveSettings, bool &bEnableDisable, NJR_ConfigData *pcd, HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (NULL != pcd)
	{
		if (!pcd->bProcessingWMScroll)
		{
			pcd->bProcessingWMScroll = true;

			HWND hWndEdit = NULL;
			bool bDivide = true;

			if      (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_BILAT_DOMAIN))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BILAT_DOMAIN); }
			else if (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_BILAT_RANGE ))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BILAT_RANGE ); }
			else if (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_1   ))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_1   ); }
			else if (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_2   ))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_2   ); }
			else if (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_3   ))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_3   ); }
			else if (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_4   ))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_4   ); }
			else if (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_BLACKPOINT  ))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BLACKPOINT  ); bDivide = false; }
			else if (lParam == reinterpret_cast<LPARAM>(GetDlgItem(hwndDlg, IDC_SPIN_BRIGHTNESS  ))) { hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BRIGHTNESS  ); }

			if (NULL != hWndEdit)
			{
				NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);

				char szEdit[64];
				if (bDivide)
				{
					double dPos = HIWORD(wParam);
					dPos /= 1000.0;
					NLeoHelpers::StringFormat(szEdit, sizeof(szEdit)/sizeof(szEdit[0]), "%.3f", dPos);
				}
				else
				{
					DWORD dwPos = HIWORD(wParam);
					NLeoHelpers::StringFormat(szEdit, sizeof(szEdit)/sizeof(szEdit[0]), "%d", dwPos);
				}
				SetWindowText(hWndEdit, szEdit);
				bResult = TRUE;
			}

			pcd->bProcessingWMScroll = false;
		}
	}
}

void NJP2RawConfig::configCommand(BOOL &bResult, bool &bDeletePCD, bool &bUpdateControls, bool &bLoadSettings, bool &bLoadDefaultSettings, bool &bSaveSettings, bool &bEnableDisable, NJR_ConfigData *pcd, HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (HIWORD(wParam) == EN_CHANGE && NULL != pcd)
	{
		if (!pcd->bProcessingWMScroll)
		{
			HWND hWndEdit = NULL;
			HWND hWndSpin = NULL;
			bool bMultiply = true;

			switch(LOWORD(wParam))
			{
			case(IDC_EDIT_BILAT_DOMAIN):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BILAT_DOMAIN);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_BILAT_DOMAIN);
				break;
			case(IDC_EDIT_BILAT_RANGE):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BILAT_RANGE);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_BILAT_RANGE);
				break;
			case(IDC_EDIT_USERMUL_1):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_1);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_1);
				break;
			case(IDC_EDIT_USERMUL_2):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_2);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_2);
				break;
			case(IDC_EDIT_USERMUL_3):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_3);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_3);
				break;
			case(IDC_EDIT_USERMUL_4):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_USERMUL_4);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_USERMUL_4);
				break;
			case(IDC_EDIT_BLACKPOINT):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BLACKPOINT);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_BLACKPOINT);
				bMultiply = false;
				break;
			case(IDC_EDIT_BRIGHTNESS):
				hWndEdit = GetDlgItem(hwndDlg, IDC_EDIT_BRIGHTNESS);
				hWndSpin = GetDlgItem(hwndDlg, IDC_SPIN_BRIGHTNESS);
				break;
			case IDC_EDIT_RAW_EXTENSION:
				bEnableDisable = true;
				bResult = TRUE;
				break;
			default:
				break;
			}

			if (NULL != hWndEdit && NULL != hWndSpin)
			{
				NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);

				char szEdit[64];
				GetWindowText(hWndEdit, szEdit, sizeof(szEdit)/sizeof(szEdit[0]));
				szEdit[sizeof(szEdit)/sizeof(szEdit[0]) - 1] = '\0';

				int iPos = 0;

				if (bMultiply)
				{
					// Convert string into number out of 1000, avoiding floating-point rounding issues by not using floating-point numbers.
					// (e.g. "atof("1.001") * 1000.0" gives the result 1000 not 1001.)

					char *szDec = strchr(szEdit, '.');
					if (NULL != szDec)
					{
						szDec = '\0';
					}
					int iPos = atoi(szEdit) * 1000;
					if (NULL != szDec)
					{
						iPos += atoi(szDec + 1);
					}
				}
				else
				{
					iPos = atoi(szEdit);
				}

				SendMessage(hWndSpin, UDM_SETPOS, 0, MAKELONG(iPos, 0));
				bResult = TRUE;
			}
		}
	}
	// Still in WM_COMMAND handler.
	if (!bResult)
	{
		switch (LOWORD(wParam))
		{
		case IDC_BUTTON_OUT_ICC:
			FilenameToEdit(pcd, hwndDlg, IDC_EDIT_OUT_ICC, "Select output ICC profile");
			break;
		case IDC_BUTTON_CAM_ICC:
			FilenameToEdit(pcd, hwndDlg, IDC_EDIT_CAM_ICC, "Select camera ICC profile");
			break;
		case IDC_LIST_CAMERAS:
			if (NULL != pcd)
			{
				if (LBN_SELCANCEL == HIWORD(wParam) || LBN_SELCHANGE == HIWORD(wParam))
				{
					StoreRawDialogData(pcd, true);
					StoreRawDialogData(pcd, false);

					int idxNewSel = (LBN_SELCANCEL == HIWORD(wParam) ? 0 : SendMessage(reinterpret_cast<HWND>(lParam), LB_GETCURSEL, 0, 0));

					pcd->strSelectedProfileSafeName = reinterpret_cast<const char *>(SendMessage(reinterpret_cast<HWND>(lParam), LB_GETITEMDATA, idxNewSel, 0));
					SetupRawDialog(pcd, true);
					SetupRawDialog(pcd, false);

					bEnableDisable = true;
				}
			}
			break;
		case IDC_BUTTON_REMOVECAMERA:
			if (NULL != pcd && !pcd->strSelectedProfileSafeName.empty())
			{
				HWND hWndList = GetDlgItem(pcd->hWndDCRawThumbs, IDC_LIST_CAMERAS);

				if (NULL != hWndList)
				{
					int idxCurSel = SendMessage(hWndList, LB_GETCURSEL, 0, 0);
					int iTotalItems = SendMessage(hWndList, LB_GETCOUNT, 0, 0);

					pcd->mapProfiles.erase(pcd->strSelectedProfileSafeName);

					if ((idxCurSel + 1) == iTotalItems)
					{
						idxCurSel--;
					}
					else
					{
						idxCurSel++;
					}

					pcd->strSelectedProfileSafeName = reinterpret_cast<const char *>(SendMessage(hWndList, LB_GETITEMDATA, idxCurSel, 0));
					SetupRawDialog(pcd, true);
					SetupRawDialog(pcd, false);

					bEnableDisable = true;
				}
			}
		case IDC_CHECK_DCRAW:
		case IDC_CHECK_TGA:
		case IDC_CHECK_JP2:
		case IDC_CHECK_PNM:
		case IDC_CHECK_RAS:
		case IDC_CHECK_BILATERAL:
		case IDC_CHECK_SETWHITE:
		case IDC_CHECK_BLACKPOINT:
			if (NULL != pcd)
			{
				NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);
				bEnableDisable = true;
				bResult = TRUE;
			}
			break;
		case IDC_COMBO_IMAGEPREFERENCE:
		case IDC_COMBO_OUT_ICC:
		case IDC_COMBO_CAM_ICC:
			if (NULL != pcd && HIWORD(wParam) == CBN_EDITCHANGE)
			{
				NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);
				bEnableDisable = true;
				bResult = TRUE;
			}
		case IDC_LIST_RAW_EXTENSIONS:
			if (NULL != pcd)
			{
				switch(HIWORD(wParam))
				{
				case LBN_SELCANCEL:
				case LBN_SELCHANGE:
					{
						HWND hwndListBox = GetDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS);
						if (NULL != hwndListBox)
						{
							bEnableDisable = true;
							bResult = TRUE;

							LRESULT selCount = SendMessage(hwndListBox, LB_GETSELCOUNT, 0, 0);
							if (0 < selCount)
							{
								int *pSelItems = new int[ selCount ];

								if (selCount == SendMessage(hwndListBox, LB_GETSELITEMS, selCount, reinterpret_cast<LPARAM>(pSelItems)))
								{
									std::basic_string<TCHAR> strLBItem;

									if (NLeoHelpers::GetListBoxItemText(hwndListBox, pSelItems[0], &strLBItem) && (!strLBItem.empty()))
									{
										SetDlgItemText(pcd->hWndFormats, IDC_EDIT_RAW_EXTENSION, strLBItem.c_str());
									}
								}

								delete [] pSelItems;
							}
						}
					}
					break;
				default:
					break;
				}
			}
			break;
		case IDC_BUTTON_REMOVE_RAW_EXTENSION:
			if (NULL != pcd)
			{
				NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);
				bEnableDisable = true;

				HWND hwndListBox = GetDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS);
				if (NULL != hwndListBox)
				{
					NLeoHelpers::EnableDlgItem(pcd->hWndFormats, LOWORD(wParam), FALSE);

					LRESULT selCount = SendMessage(hwndListBox, LB_GETSELCOUNT, 0, 0);
					if (0 < selCount)
					{
						int *pSelItems = new int[ selCount ];

						if (selCount == SendMessage(hwndListBox, LB_GETSELITEMS, selCount, reinterpret_cast<LPARAM>(pSelItems)))
						{
							int iDeleteCount = 0;

							for(LRESULT i = 0; i < selCount; i++)
							{
								if (0 < SendMessage(hwndListBox, LB_DELETESTRING, pSelItems[i] - iDeleteCount, 0))
								{
									iDeleteCount++;
								}
								else
								{
									break;
								}
							}
						}

						delete [] pSelItems;
					}
				}
			}
			break;
		case IDC_BUTTON_ADD_RAW_EXTENSION:
			if (NULL != pcd)
			{
				LONG lStyleDisabled = WS_DISABLED;
				LONG lButtonStyle;

				if ((!NLeoHelpers::GetDlgItemLong(pcd->hWndFormats, IDC_BUTTON_ADD_RAW_EXTENSION, GWL_STYLE, &lButtonStyle))
				||	(lButtonStyle & lStyleDisabled))
				{
					MessageBeep(-1);
				}
				else
				{
					NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);
					bEnableDisable = true;

					std::basic_string< TCHAR > strText;
					if (NLeoHelpers::GetDlgItemText(pcd->hWndFormats, IDC_EDIT_RAW_EXTENSION, &strText) && strText.length() > 0)
					{
						if (strText[0] != '.')
						{
							strText.insert(0, ".");
						}

						HWND hwndListBox = GetDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS);
						if (NULL != hwndListBox)
						{
							SendMessage(hwndListBox, LB_ADDSTRING, 0, reinterpret_cast<LPARAM>(strText.c_str()));
							SetDlgItemText(pcd->hWndFormats, IDC_EDIT_RAW_EXTENSION, "");
						}
					}
				}
			}
			break;
		case IDC_BUTTON_DEFAULTS:
			if (NULL != pcd)
			{
				if (IDOK == MessageBox(pcd->hWndMainDlg, "This will reset all configuration pages, including the removal of any customized camera profiles.", "Warning", MB_OKCANCEL|MB_ICONEXCLAMATION))
				{
					NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);
					bLoadDefaultSettings = true;
					bResult = TRUE;
				}
			}
			break;
		case IDC_BUTTON_APPLY:
			if (NULL != pcd)
			{
				//NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, FALSE);
				bSaveSettings = true;
				bResult = TRUE;
			}
			break;
		case IDOK:
			{
				HWND hwndFocus = GetFocus();
				
				if (NULL != hwndFocus && NULL != pcd)
				{
					if (hwndFocus == GetDlgItem(pcd->hWndFormats, IDC_EDIT_RAW_EXTENSION))
					{
						SendMessage(pcd->hWndFormats, WM_COMMAND, IDC_BUTTON_ADD_RAW_EXTENSION, reinterpret_cast<LPARAM>(GetDlgItem(pcd->hWndFormats, IDC_BUTTON_ADD_RAW_EXTENSION)));
						bResult = TRUE;
						break;
					}
				}

				bSaveSettings = true;
			}
			// *** FALL THROUGH FROM IDOK TO IDCANCEL *** //
		case IDCANCEL:
			{
				bDeletePCD = true;
				PostQuitMessage(0);
				bResult = TRUE;
			}
			break;
		default:
			break;
		}
	}
}

void NJP2RawConfig::configDropFiles(BOOL &bResult, bool &bDeletePCD, bool &bUpdateControls, bool &bLoadSettings, bool &bLoadDefaultSettings, bool &bSaveSettings, bool &bEnableDisable, NJR_ConfigData *pcd, HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (NULL != pcd && 0 == wParam
	&&	(lParam == reinterpret_cast<LPARAM>(GetDlgItem(pcd->hWndDCRawThumbs, IDC_LIST_CAMERAS))
	||	 lParam == reinterpret_cast<LPARAM>(GetDlgItem(pcd->hWndDCRawViewers, IDC_LIST_CAMERAS))))
	{
		// Note: This non-standard WM_DROPFILES is sent from our CDropTarget class, not from the OS. i.e. We are re-using the window message.

		StoreRawDialogData(pcd, true);
		StoreRawDialogData(pcd, false);

		std::vector< std::basic_string<TCHAR> > &vecFilenames = pcd->dropTarget.GetFilenames();

		int nTotalFiles = vecFilenames.size();
		int nRecognisedFiles = 0;

		for (std::vector< std::basic_string<TCHAR> >::iterator pFilename = vecFilenames.begin(); pFilename != vecFilenames.end(); ++pFilename)
		{
			std::basic_string<TCHAR>::size_type filenameLength = pFilename->length();

			if (filenameLength >= 9 && 0 == _stricmp("dopus.exe", (pFilename->c_str() + filenameLength) - 9))
			{
				nRecognisedFiles++;

				if (!pcd->bWithTongue)
				{
					HWND hWndTab = GetDlgItem(pcd->hWndMainDlg, IDC_TAB);

					if (NULL != hWndTab)
					{
						TCITEM tabItem;
						ZeroMemory(&tabItem, sizeof(tabItem));
						tabItem.mask = TCIF_PARAM|TCIF_TEXT;

						tabItem.pszText = "With Tongue";
						tabItem.lParam = reinterpret_cast<LPARAM>(pcd->hWndWithTongue);
						SendMessage(hWndTab, TCM_INSERTITEM, TabCtrl_GetItemCount(hWndTab), reinterpret_cast<LPARAM>(&tabItem));

						// Stupid tab control paints over windows on top of it when you add a tab.
						InvalidateRect(pcd->hWndFormats,      NULL, TRUE);
						InvalidateRect(pcd->hWndDCRawThumbs,  NULL, TRUE);
						InvalidateRect(pcd->hWndDCRawViewers, NULL, TRUE);

						// Make the tongue dialog decode and show the initial image.
						SendMessage(pcd->hWndWithTongue, WM_VSCROLL, 0, reinterpret_cast<LPARAM>(GetDlgItem(pcd->hWndWithTongue, IDC_SLIDER_TONGUE)));

						pcd->bWithTongue = true;
					}
				}
			}
			else
			{
				CWin32IOFileWrapper fileWrapper(pFilename->c_str(), NULL);
				fileWrapper.SetThrowExceptions(false);

				std::string strMake;
				std::string strModel;

				std::vector< char * > argvec;
				argvec.push_back("dcraw.exe");	// Executable name by convention. Not going to be used.
				argvec.push_back(""); // Requires empty string at the end. Do not count in argc argument.

				// NULL settings and output name means we just identify the file (quick) and don't decode the image inside it (slow).
				DCRaw dcr(pcd->pLcmsCS, NULL);
				if (dcr.DoFile(NULL, &fileWrapper, pFilename->c_str(), NULL, &strMake, &strModel, NULL, NULL))
				{
					std::string strDisplayName = NJR_ProfileMap::GenerateDisplayName(strMake.c_str(), strModel.c_str());

					if (!strDisplayName.empty())
					{
						std::string strSafeName = NLeoHelpers::GenerateSafeName(strDisplayName.c_str());

						nRecognisedFiles++;

						HWND hWndList = GetDlgItem(pcd->hWndDCRawThumbs, IDC_LIST_CAMERAS);
						if (NULL != hWndList)
						{
							const char *szCurrentProfileSafeName = reinterpret_cast<const char *>(SendMessage(hWndList, LB_GETITEMDATA, SendMessage(hWndList, LB_GETCURSEL, 0, 0), 0));

							NJR_ProfileMap::iterator pCurrentRawProfile = pcd->mapProfiles.find(szCurrentProfileSafeName);

							NJR_ProfileMap::iterator pNewRawProfile = pcd->mapProfiles.find(strSafeName.c_str());

							if (pNewRawProfile == pcd->mapProfiles.end())
							{
								pNewRawProfile = pcd->mapProfiles.insert(std::pair< std::string, NJR_RawProfile >(strSafeName, pCurrentRawProfile->second)).first;
								pNewRawProfile->second.strDisplayName = strDisplayName;
								pNewRawProfile->second.strSafeName = strSafeName;

								NLeoHelpers::EnableDlgItem(pcd->hWndMainDlg, IDC_BUTTON_APPLY, TRUE);
							}

							pcd->strSelectedProfileSafeName = strSafeName;
							SetupRawDialog(pcd, true);
							SetupRawDialog(pcd, false);

							bEnableDisable = true;
						}
					}
				}
			}
		}

		vecFilenames.clear();

		if (nRecognisedFiles != nTotalFiles)
		{
			const char *szMessage = "Not all dropped files were recognised as raw camera images.";

			if (nTotalFiles == 1)
			{
				szMessage = "The dropped file was not recognised as a raw camera image.";
			}
			else if (nRecognisedFiles == 0)
			{
				szMessage = "None of the dropped files were recognised as raw camera images.";
			}

			MessageBox(pcd->hWndMainDlg, szMessage, "DCRaw Decoder", MB_OK|MB_ICONWARNING);
		}
	}
}

void NJP2RawConfig::configLoadSettings(NJR_ConfigData *pcd, bool bLoadDefaultSettings, bool bChangeSelections)
{
	DWORD dwFlags = RegGetFlags(bLoadDefaultSettings);

	CheckDlgButton(pcd->hWndFormats, IDC_CHECK_DCRAW, (dwFlags&NJRF_DISABLE_DCRAW) ? BST_UNCHECKED : BST_CHECKED);
	CheckDlgButton(pcd->hWndFormats, IDC_CHECK_TGA,   (dwFlags&NJRF_DISABLE_TGA)   ? BST_UNCHECKED : BST_CHECKED);
	CheckDlgButton(pcd->hWndFormats, IDC_CHECK_JP2,   (dwFlags&NJRF_DISABLE_JP2)   ? BST_UNCHECKED : BST_CHECKED);
	CheckDlgButton(pcd->hWndFormats, IDC_CHECK_PNM,   (dwFlags&NJRF_DISABLE_PNM)   ? BST_UNCHECKED : BST_CHECKED);
	CheckDlgButton(pcd->hWndFormats, IDC_CHECK_RAS,   (dwFlags&NJRF_DISABLE_RAS)   ? BST_UNCHECKED : BST_CHECKED);
	CheckDlgButton(pcd->hWndFormats, IDC_CHECK_TIF,   (dwFlags&NJRF_DISABLE_TIF)   ? BST_UNCHECKED : BST_CHECKED);

	std::vector< std::basic_string<TCHAR> > vecStrings;
	HWND hwndListBox;

	if (NULL != (hwndListBox = GetDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS)))
	{
		for (LRESULT lr = SendMessage(hwndListBox, LB_GETCOUNT, 0, 0); 0 < lr; lr = SendMessage(hwndListBox, LB_DELETESTRING, 0, 0))
		{
		}

		RegGetRawExtensions(bLoadDefaultSettings, &vecStrings);
		std::sort( vecStrings.begin(), vecStrings.end() );
		for (std::vector< std::basic_string<TCHAR> >::const_iterator pStr = vecStrings.begin(); pStr != vecStrings.end(); ++pStr)
		{
			SendMessage(hwndListBox, LB_ADDSTRING, 0, reinterpret_cast<LPARAM>(pStr->c_str()));
		}
	}

	RegGetRawProfiles(bLoadDefaultSettings, pcd->mapProfiles);

	if (bChangeSelections)
	{
		pcd->strActivePage.clear();
		pcd->strSelectedProfileSafeName.clear();

		if (!bLoadDefaultSettings)
		{
			NLeoHelpers::LeetRegQueryStringValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("ConfigPage"),       &pcd->strActivePage);
			NLeoHelpers::LeetRegQueryStringValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("ConfigRawProfile"), &pcd->strSelectedProfileSafeName);
		}

		HWND hWndTab = GetDlgItem(pcd->hWndMainDlg, IDC_TAB);
		if (NULL != hWndTab)
		{
			int activeTab = 0;

			if      (0 == stricmp(pcd->strActivePage.c_str(), "Formats"   )) { activeTab = 0; }
			else if (0 == stricmp(pcd->strActivePage.c_str(), "RawThumbs" )) { activeTab = 1; }
			else if (0 == stricmp(pcd->strActivePage.c_str(), "RawViewers")) { activeTab = 2; }

			TabCtrl_SetCurSel(hWndTab, activeTab);

			ShowWindow(pcd->hWndFormats,      (activeTab == 0) ? SW_SHOW : SW_HIDE);
			ShowWindow(pcd->hWndDCRawThumbs,  (activeTab == 1) ? SW_SHOW : SW_HIDE);
			ShowWindow(pcd->hWndDCRawViewers, (activeTab == 2) ? SW_SHOW : SW_HIDE);
			ShowWindow(pcd->hWndWithTongue,   (activeTab == 3) ? SW_SHOW : SW_HIDE);
		}
	}

	SetupRawDialog(pcd, true);
	SetupRawDialog(pcd, false);
}

void NJP2RawConfig::configSaveSettings(NJR_ConfigData *pcd)
{
	DWORD dwFlags = 0;

	dwFlags |= ((BST_UNCHECKED == IsDlgButtonChecked(pcd->hWndFormats, IDC_CHECK_DCRAW)) ? NJRF_DISABLE_DCRAW : 0);
	dwFlags |= ((BST_UNCHECKED == IsDlgButtonChecked(pcd->hWndFormats, IDC_CHECK_TGA  )) ? NJRF_DISABLE_TGA   : 0);
	dwFlags |= ((BST_UNCHECKED == IsDlgButtonChecked(pcd->hWndFormats, IDC_CHECK_JP2  )) ? NJRF_DISABLE_JP2   : 0);
	dwFlags |= ((BST_UNCHECKED == IsDlgButtonChecked(pcd->hWndFormats, IDC_CHECK_PNM  )) ? NJRF_DISABLE_PNM   : 0);
	dwFlags |= ((BST_UNCHECKED == IsDlgButtonChecked(pcd->hWndFormats, IDC_CHECK_RAS  )) ? NJRF_DISABLE_RAS   : 0);
	dwFlags |= ((BST_UNCHECKED == IsDlgButtonChecked(pcd->hWndFormats, IDC_CHECK_TIF  )) ? NJRF_DISABLE_TIF   : 0);

	StoreRawDialogData(pcd, true);
	StoreRawDialogData(pcd, false);

	NLeoHelpers::LeetRegSetDWORDValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("Flags"), dwFlags);

	std::vector< std::basic_string<TCHAR> > vecStrings;
	HWND hwndListBox;

	if (NULL != (hwndListBox = GetDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS)))
	{
		LRESULT itemCount = SendMessage(hwndListBox, LB_GETCOUNT, 0, 0);
		if (0 <= itemCount)
		{
			vecStrings.clear();
			for (LRESULT i = 0; i < itemCount; i++)
			{
				std::basic_string<TCHAR> strLBItem;

				if (NLeoHelpers::GetListBoxItemText(hwndListBox, i, &strLBItem) && (!strLBItem.empty()))
				{
					vecStrings.push_back( strLBItem );
				}
			}
			NLeoHelpers::LeetRegSetMultiStringValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("RawExtensions"), &vecStrings);
		}
	}

	RegSaveRawProfiles(pcd->mapProfiles);

	NLeoHelpers::LeetRegSetStringValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("ConfigPage"),       pcd->strActivePage.c_str());
	NLeoHelpers::LeetRegSetStringValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("ConfigRawProfile"), pcd->strSelectedProfileSafeName.c_str());

	HWND  hWndNotify	= (NULL == pcd) ? NULL : pcd->hWndNotify;
	DWORD dwNotifyData	= (NULL == pcd) ? 0    : pcd->dwNotifyData;

	if (NULL != hWndNotify)
	{
		PostMessage(hWndNotify, DVPLUGINMSG_THUMBSCHANGED, DVPTCF_REDRAW | DVPTCF_FLUSHCACHE, 0);
		PostMessage(hWndNotify, DVPLUGINMSG_REINITIALIZE, 0, dwNotifyData);
	}
}

void NJP2RawConfig::configEnableDisable(NJR_ConfigData *pcd)
{
	BOOL bRawEnabled = (BST_CHECKED == IsDlgButtonChecked(pcd->hWndFormats, IDC_CHECK_DCRAW) ? TRUE : FALSE);

	NLeoHelpers::EnableDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS, bRawEnabled);
	NLeoHelpers::EnableDlgItem(pcd->hWndFormats, IDC_EDIT_RAW_EXTENSION, bRawEnabled);

	// Remove Raw Extension button
	HWND hwndListBox = GetDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS);
	if (NULL != hwndListBox)
	{
		LRESULT selCount = SendMessage(hwndListBox, LB_GETSELCOUNT, 0, 0);
		if (0 >= selCount)
		{
			NLeoHelpers::EnableDlgItem(pcd->hWndFormats, IDC_BUTTON_REMOVE_RAW_EXTENSION, FALSE);
		}
		else
		{
			NLeoHelpers::EnableDlgItem(pcd->hWndFormats, IDC_BUTTON_REMOVE_RAW_EXTENSION, bRawEnabled);
		}
	}

	// Add Raw Extension button
	std::basic_string< TCHAR > strText;
	if (NLeoHelpers::GetDlgItemText(pcd->hWndFormats, IDC_EDIT_RAW_EXTENSION, &strText))
	{
		HWND hwndListBox = GetDlgItem(pcd->hWndFormats, IDC_LIST_RAW_EXTENSIONS);
		if (NULL != hwndListBox)
		{
			if (strText.length() > 0 && strText[0] != '.')
			{
				strText.insert(0, ".");
			}

			BOOL bNotFound = TRUE;
			if (strText.empty()
			||	0 <= SendMessage(hwndListBox, LB_FINDSTRINGEXACT, -1, reinterpret_cast<LPARAM>(strText.c_str())))
			{
				bNotFound = FALSE;
			}
			NLeoHelpers::EnableDlgItem(pcd->hWndFormats, IDC_BUTTON_ADD_RAW_EXTENSION, bRawEnabled && bNotFound);
		}
	}

	// DCRaw dialogs
	EnableDisableRawDialog(pcd, bRawEnabled, true);
	EnableDisableRawDialog(pcd, bRawEnabled, false);
}

void NJP2RawConfig::SetupRawDialog(NJR_ConfigData *pcd, bool bThumbs)
{
	HWND hWndDCRaw = bThumbs ? pcd->hWndDCRawThumbs : pcd->hWndDCRawViewers;

	HWND hWndList = GetDlgItem(hWndDCRaw, IDC_LIST_CAMERAS);

	bool bLoadedOne = false;
	bool bAddedOne = false;

	if (NULL != hWndList)
	{
		SendMessage(hWndList, LB_RESETCONTENT, 0, 0);

		for (NJR_ProfileMap::iterator pPair = pcd->mapProfiles.begin(); pPair != pcd->mapProfiles.end(); ++pPair)
		{
			LRESULT idx = SendMessage(hWndList, LB_ADDSTRING, 0, reinterpret_cast<LPARAM>(pPair->second.strDisplayName.c_str()));

			if (idx >= 0)
			{
				SendMessage(hWndList, LB_SETITEMDATA, idx, reinterpret_cast<LPARAM>(pPair->second.strSafeName.c_str()));
				bAddedOne = true;

				if (0 == strcmp(pPair->second.strSafeName.c_str(), pcd->strSelectedProfileSafeName.c_str()))
				{
					SendMessage(hWndList, LB_SETCURSEL, idx, 0);
					SetupRawProfileInDialog(pcd, pPair->second, bThumbs);
					bLoadedOne = true;
				}
			}
		}

		if (bAddedOne && !bLoadedOne)
		{
			SendMessage(hWndList, LB_SETCURSEL, 0, 0);
			SetupRawProfileInDialog(pcd, pcd->mapProfiles.begin()->second, bThumbs);
		}
	}
}

void NJP2RawConfig::SetupRawProfileInDialog(NJR_ConfigData *pcd, NJR_RawProfile &rawProfile, bool bThumbs)
{
	HWND hWndDCRaw = bThumbs ? pcd->hWndDCRawThumbs : pcd->hWndDCRawViewers;

	NJR_RawSettings &rs = bThumbs ? rawProfile.rsThumbs : rawProfile.rsViewers;

	HWND hWndComboImagePreferences	= GetDlgItem(hWndDCRaw, IDC_COMBO_IMAGEPREFERENCE);

	int iDISelect = (rs.bTryPreview ? CF_DI_PREVIEW : 0) | (rs.bTryFull ? CF_DI_FULL : 0);
	SetupComboItem(hWndDCRaw, IDC_COMBO_DISPLAYEDIMAGE, true,  0,                        iDISelect, "Disabled");
	SetupComboItem(hWndDCRaw, IDC_COMBO_DISPLAYEDIMAGE, false, CF_DI_PREVIEW,            iDISelect, "Preview (fast) if available");
	SetupComboItem(hWndDCRaw, IDC_COMBO_DISPLAYEDIMAGE, false, CF_DI_PREVIEW|CF_DI_FULL, iDISelect, "Preview (fast) if available, else Full (slow)");
	SetupComboItem(hWndDCRaw, IDC_COMBO_DISPLAYEDIMAGE, false, CF_DI_FULL,               iDISelect, "Full (slow)");

	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, true,   -1, rs.iRotation, "Automatic");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false,   0, rs.iRotation, "None");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false,  90, rs.iRotation, "90 CW");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false, 180, rs.iRotation, "180");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false, 270, rs.iRotation, "270 CW");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false,   1, rs.iRotation, "Flip Horizontal");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false,   2, rs.iRotation, "Flip Vertical");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false,   4, rs.iRotation, "Transpose");
	SetupComboItem(hWndDCRaw, IDC_COMBO_ROTATION, false,   7, rs.iRotation, "Horizontal + Vertical + Transpose");

	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_FUJITURN, rs.bFujiTurn);

	SetupComboItem(hWndDCRaw, IDC_COMBO_INTERPQUALITY, true,  -1, rs.iInterpQuality, "Default");
	SetupComboItem(hWndDCRaw, IDC_COMBO_INTERPQUALITY, false,  0, rs.iInterpQuality, "Bilinear (high speed, low quality)");
//	SetupComboItem(hWndDCRaw, IDC_COMBO_INTERPQUALITY, false,  1, rs.iInterpQuality, "dcraw -q1");
	SetupComboItem(hWndDCRaw, IDC_COMBO_INTERPQUALITY, false,  2, rs.iInterpQuality, "VNG: Variable Number of Gradients");
	SetupComboItem(hWndDCRaw, IDC_COMBO_INTERPQUALITY, false,  3, rs.iInterpQuality, "AHD: Adaptive Homogeneity-Directed");

	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_DOCMODE_NOCOL, rs.bDocModeNoCol);
	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_DOCMODE_RAW,   rs.bDocModeRaw);
	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_HALFSIZECOLOR, rs.bHalfSizeColor);
	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_RGGB,          rs.bRGGB);
	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_USESECONDARY,  rs.bUseSecondary);

	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_BILATERAL,     rs.bBilateral);
	SetupSpinItem(hWndDCRaw, IDC_SPIN_BILAT_DOMAIN, 10000, 1, rs.iBilatDomain); // Min/Max chosen somewhat arbitrarily.
	SetupSpinItem(hWndDCRaw, IDC_SPIN_BILAT_RANGE,  10000, 1, rs.iBilatRange);  // Min/Max chosen somewhat arbitrarily.

	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_AUTOWHITE,   rs.bAutoWhite);
	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_CAMERAWHITE, rs.bCamWhite);
	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_SETWHITE,    rs.bSetWhite);
	SetupSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_1, 10000, 1, rs.iUserMul1); // Min/Max chosen somewhat arbitrarily.
	SetupSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_2, 10000, 1, rs.iUserMul2); // Min/Max chosen somewhat arbitrarily.
	SetupSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_3, 10000, 1, rs.iUserMul3); // Min/Max chosen somewhat arbitrarily.
	SetupSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_4, 10000, 1, rs.iUserMul4); // Min/Max chosen somewhat arbitrarily.

	SetupCheckBoxItem(hWndDCRaw, IDC_CHECK_BLACKPOINT, rs.bSetBlackPoint);
	SetupSpinItem(hWndDCRaw, IDC_SPIN_BLACKPOINT, 10000, 0, rs.iBlackPoint); // Max chosen somewhat arbitrarily.

	SetupSpinItem(hWndDCRaw, IDC_SPIN_BRIGHTNESS, 10000, 1, rs.iBrightness); // Min/Max chosen somewhat arbitrarily.

	SetupComboItem(hWndDCRaw, IDC_COMBO_HIGHLIGHTMODE, true,  0, rs.iHighlightMode, "Clip all to solid white");
	SetupComboItem(hWndDCRaw, IDC_COMBO_HIGHLIGHTMODE, false, 1, rs.iHighlightMode, "Leave unclipped in shades of pink");
	SetupComboItem(hWndDCRaw, IDC_COMBO_HIGHLIGHTMODE, false, 2, rs.iHighlightMode, "Attempt to intelligently reconstruct");

	SetupComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, true,   0, rs.iOutIccType, "Raw color (unique to each camera)");
	SetupComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, false,  1, rs.iOutIccType, "sRGB D65 (default)");
	SetupComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, false,  2, rs.iOutIccType, "Adobe 1998 D65");
	SetupComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, false,  3, rs.iOutIccType, "Wide Gamut D65");
	SetupComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, false,  4, rs.iOutIccType, "Kodak ProPhoto D65");
	SetupComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, false,  5, rs.iOutIccType, "XYZ");
	SetupComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, false, -1, rs.iOutIccType, "Load ICC profile from file...");

	SetupEditItem(hWndDCRaw, IDC_EDIT_OUT_ICC, rs.strOutIcc.c_str());

	SetupComboItem(hWndDCRaw, IDC_COMBO_CAM_ICC, true,   0, rs.iCamIccType, "None");
	SetupComboItem(hWndDCRaw, IDC_COMBO_CAM_ICC, false,  1, rs.iCamIccType, "Embedded profile in raw files");
	SetupComboItem(hWndDCRaw, IDC_COMBO_CAM_ICC, false, -1, rs.iCamIccType, "Load ICC profile from file...");

	SetupEditItem(hWndDCRaw, IDC_EDIT_CAM_ICC, rs.strCamIcc.c_str());
}

void NJP2RawConfig::StoreRawDialogData(NJR_ConfigData *pcd, bool bThumbs)
{
	HWND hWndDCRaw = bThumbs ? pcd->hWndDCRawThumbs : pcd->hWndDCRawViewers;

	HWND hWndList = GetDlgItem(hWndDCRaw, IDC_LIST_CAMERAS);

	if (NULL != hWndList)
	{
		const char *szProfileSafeName = pcd->strSelectedProfileSafeName.c_str(); // reinterpret_cast<const char *>(SendMessage(hWndList, LB_GETITEMDATA, SendMessage(hWndList, LB_GETCURSEL, 0, 0), 0));

		NJR_ProfileMap::iterator pRawProfile = pcd->mapProfiles.find(szProfileSafeName);

		if (pRawProfile != pcd->mapProfiles.end())
		{
			NJR_RawSettings &rs = bThumbs ? pRawProfile->second.rsThumbs : pRawProfile->second.rsViewers;

			rs = NJR_RawSettings(); // Overwrite with defaults.

			int iDISelect = CF_DI_PREVIEW;
			GetComboItem(hWndDCRaw, IDC_COMBO_DISPLAYEDIMAGE, iDISelect);
			rs.bTryPreview = ((iDISelect & CF_DI_PREVIEW) ? true : false);
			rs.bTryFull    = ((iDISelect & CF_DI_FULL)    ? true : false);

			GetComboItem(hWndDCRaw, IDC_COMBO_ROTATION, rs.iRotation);
			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_FUJITURN, rs.bFujiTurn);

			GetComboItem(hWndDCRaw, IDC_COMBO_INTERPQUALITY, rs.iInterpQuality);

			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_DOCMODE_NOCOL, rs.bDocModeNoCol);
			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_DOCMODE_RAW,   rs.bDocModeRaw);
			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_HALFSIZECOLOR, rs.bHalfSizeColor);
			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_RGGB,          rs.bRGGB);
			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_USESECONDARY,  rs.bUseSecondary);

			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_BILATERAL,     rs.bBilateral);
			GetSpinItem(hWndDCRaw, IDC_SPIN_BILAT_DOMAIN, rs.iBilatDomain);
			GetSpinItem(hWndDCRaw, IDC_SPIN_BILAT_RANGE,  rs.iBilatRange);

			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_AUTOWHITE,   rs.bAutoWhite);
			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_CAMERAWHITE, rs.bCamWhite);
			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_SETWHITE,    rs.bSetWhite);
			GetSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_1, rs.iUserMul1);
			GetSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_2, rs.iUserMul2);
			GetSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_3, rs.iUserMul3);
			GetSpinItem(hWndDCRaw, IDC_SPIN_USERMUL_4, rs.iUserMul4);

			GetCheckBoxItem(hWndDCRaw, IDC_CHECK_BLACKPOINT, rs.bSetBlackPoint);
			GetSpinItem(hWndDCRaw, IDC_SPIN_BLACKPOINT, rs.iBlackPoint);
			GetSpinItem(hWndDCRaw, IDC_SPIN_BRIGHTNESS, rs.iBrightness);

			GetComboItem(hWndDCRaw, IDC_COMBO_HIGHLIGHTMODE, rs.iHighlightMode);

			GetComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, rs.iOutIccType);
			GetEditItem(hWndDCRaw, IDC_EDIT_OUT_ICC, rs.strOutIcc);

			GetComboItem(hWndDCRaw, IDC_COMBO_CAM_ICC, rs.iCamIccType);
			GetEditItem(hWndDCRaw, IDC_EDIT_CAM_ICC, rs.strCamIcc);
		}
	}
}

void NJP2RawConfig::EnableDisableRawDialog(NJR_ConfigData *pcd, BOOL bRawEnabled, bool bThumbs)
{
	HWND hWndDCRaw = bThumbs ? pcd->hWndDCRawThumbs : pcd->hWndDCRawViewers;

	int iTemp;
	bool bTemp;

	iTemp = CF_DI_PREVIEW;
	GetComboItem(hWndDCRaw, IDC_COMBO_DISPLAYEDIMAGE, iTemp);
	BOOL bTryPreview = bRawEnabled && ((iTemp & CF_DI_PREVIEW) ? true : false);
	BOOL bTryFull    = bRawEnabled && ((iTemp & CF_DI_FULL)    ? true : false);

	bTemp = false; GetCheckBoxItem(hWndDCRaw, IDC_CHECK_BILATERAL,  bTemp); BOOL bBilat      = bTemp ? TRUE : FALSE;
	bTemp = false; GetCheckBoxItem(hWndDCRaw, IDC_CHECK_SETWHITE,   bTemp); BOOL bSetWhite   = bTemp ? TRUE : FALSE;
	bTemp = false; GetCheckBoxItem(hWndDCRaw, IDC_CHECK_BLACKPOINT, bTemp); BOOL bBlackpoint = bTemp ? TRUE : FALSE;

	iTemp = 1; GetComboItem(hWndDCRaw, IDC_COMBO_OUT_ICC, iTemp); BOOL bOutIccFile = iTemp == -1 ? TRUE : FALSE;
	iTemp = 0; GetComboItem(hWndDCRaw, IDC_COMBO_CAM_ICC, iTemp); BOOL bCamIccFile = iTemp == -1 ? TRUE : FALSE;

	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_CAMERAS,				bRawEnabled);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_LIST_CAMERAS,					bRawEnabled);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_BUTTON_REMOVECAMERA,			(bRawEnabled && !pcd->strSelectedProfileSafeName.empty() ? TRUE : FALSE));
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_DROPMSG,				bRawEnabled);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_DISPLAYEDIMAGE,		bRawEnabled);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_COMBO_DISPLAYEDIMAGE,			bRawEnabled);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_ROTATION,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_COMBO_ROTATION,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_FUJITURN,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_INTERPQUALITY,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_COMBO_INTERPQUALITY,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_DOCMODE_NOCOL,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_DOCMODE_RAW,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_HALFSIZECOLOR,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_RGGB,					bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_USESECONDARY,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_BILATERAL,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_BILAT_DOMAIN,			bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_BILAT_DOMAIN,			bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_BILAT_DOMAIN,			bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_BILAT_DOMAIN_UNITS,	bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_BILAT_RANGE,			bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_BILAT_RANGE,				bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_BILAT_RANGE,				bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_BILAT_RANGE_UNITS,		bTryFull && bBilat);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_AUTOWHITE,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_CAMERAWHITE,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_SETWHITE,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_USERMUL_1,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_USERMUL_1,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_USERMUL_2,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_USERMUL_2,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_USERMUL_3,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_USERMUL_3,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_USERMUL_4,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_USERMUL_4,				bTryFull && bSetWhite);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_CHECK_BLACKPOINT,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_BLACKPOINT,				bTryFull && bBlackpoint);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_BLACKPOINT,				bTryFull && bBlackpoint);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_BRIGHTNESS,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_BRIGHTNESS,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_SPIN_BRIGHTNESS,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_HIGHLIGHTMODE,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_COMBO_HIGHLIGHTMODE,			bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_OUT_ICC,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_COMBO_OUT_ICC,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_OUT_ICC,					bTryFull && bOutIccFile);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_BUTTON_OUT_ICC,				bTryFull && bOutIccFile);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_STATIC_CAM_ICC,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_COMBO_CAM_ICC,				bTryFull);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_EDIT_CAM_ICC,					bTryFull && bCamIccFile);
	NLeoHelpers::EnableDlgItem(hWndDCRaw, IDC_BUTTON_CAM_ICC,				bTryFull && bCamIccFile);
}

void NJP2RawConfig::SetupComboItem(HWND hWndParent, int iDlgItem, bool bClear, int iValue, int iSelectValue, const char *szTitle)
{
	HWND hWndCombo = GetDlgItem(hWndParent, iDlgItem);

	if (hWndCombo != NULL)
	{
		if (bClear)
		{
			SendMessage(hWndCombo, CB_RESETCONTENT, 0, 0);
		}

		LRESULT idx = SendMessage(hWndCombo, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(szTitle));
		if (0 <= idx)
		{
			DWORD dwValue = *reinterpret_cast<DWORD *>(&iValue);

			SendMessage(hWndCombo, CB_SETITEMDATA, idx, dwValue);

			if (iSelectValue == iValue)
			{
				SendMessage(hWndCombo, CB_SETCURSEL, idx, 0);
			}
		}
	}
}

void NJP2RawConfig::GetComboItem(HWND hWndParent, int iDlgItem, int &iValue)
{
	HWND hWndCombo = GetDlgItem(hWndParent, iDlgItem);

	if (hWndCombo != NULL)
	{
		LRESULT curSel = SendMessage(hWndCombo, CB_GETCURSEL, 0, 0);

		DWORD dwValue = SendMessage(hWndCombo, CB_GETITEMDATA, curSel, 0);

		iValue = *reinterpret_cast<int *>(&dwValue);
	}
}

void NJP2RawConfig::SetupCheckBoxItem(HWND hWndParent, int iDlgItem, bool bCheck)
{
	CheckDlgButton(hWndParent, iDlgItem, bCheck ? BST_CHECKED : BST_UNCHECKED);
}

void NJP2RawConfig::GetCheckBoxItem(HWND hWndParent, int iDlgItem, bool &bValue)
{
	bValue = (BST_CHECKED == IsDlgButtonChecked(hWndParent, iDlgItem));
}

void NJP2RawConfig::SetupSpinItem(HWND hWndParent, int iDlgItem, int iMax, int iMin, int iValue)
{
	HWND hWndSpin = GetDlgItem(hWndParent, iDlgItem);

	if (hWndSpin != NULL)
	{
		SendMessage(hWndSpin, UDM_SETRANGE, 0, MAKELONG(iMax, iMin));
		SendMessage(hWndSpin, UDM_SETPOS,   0, MAKELONG(iValue, 0));
		SendMessage(hWndParent, WM_VSCROLL, MAKELONG(0, iValue), reinterpret_cast<LPARAM>(hWndSpin));
	}
}

void NJP2RawConfig::GetSpinItem(HWND hWndParent, int iDlgItem, int &iValue)
{
	HWND hWndSpin = GetDlgItem(hWndParent, iDlgItem);

	if (hWndSpin != NULL)
	{
		iValue = LOWORD(SendMessage(hWndSpin, UDM_GETPOS, 0, 0));
	}
}

void NJP2RawConfig::SetupEditItem(HWND hWndParent, int iDlgItem, const char *szText)
{
	HWND hWndEdit = GetDlgItem(hWndParent, iDlgItem);

	if (hWndEdit != NULL)
	{
		SetWindowText(hWndEdit, szText);
	}
}

void NJP2RawConfig::GetEditItem(HWND hWndParent, int iDlgItem, std::string &strText)
{
	NLeoHelpers::GetDlgItemText(hWndParent, iDlgItem, &strText);
}

void NJP2RawConfig::FilenameToEdit(NJR_ConfigData *pcd, HWND hwndDlg, int iDlgItem, char *szTitle)
{
	TCHAR szBuffer[_MAX_PATH];
	std::string strCurrent;
	NLeoHelpers::GetDlgItemText(hwndDlg, iDlgItem, &strCurrent);
	NLeoHelpers::StringCopy(szBuffer, strCurrent.c_str(), sizeof(szBuffer)/sizeof(szBuffer[0]));

	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = pcd->hWndMainDlg;
	ofn.hInstance = pcd->hInstance;
	ofn.lpstrFile = szBuffer;
	ofn.nMaxFile = sizeof(szBuffer)/sizeof(szBuffer[0]);
	ofn.lpstrTitle = szTitle;
	ofn.Flags = OFN_EXTENSIONDIFFERENT|OFN_FILEMUSTEXIST|OFN_NOCHANGEDIR|OFN_LONGNAMES;

	if (GetOpenFileName(&ofn))
	{
		SetDlgItemText(hwndDlg, iDlgItem, ofn.lpstrFile);
	}
}


DWORD NJP2RawConfig::RegGetFlags(bool bGetDefaultValue)
{
	DWORD dwFlags;

	if (bGetDefaultValue
	||	(!NLeoHelpers::LeetRegQueryDWORDValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("Flags"), &dwFlags)))
	{
		dwFlags = 0;
	}

	return(dwFlags);
}

void NJP2RawConfig::RegGetRawExtensions(bool bGetDefaultValue, std::vector< std::basic_string<TCHAR> > *pvecResult)
{
	if (bGetDefaultValue
	||	(!NLeoHelpers::LeetRegQueryMultiStringValue(HKEY_CURRENT_USER, JP2RAW_REG_PATH, _T("RawExtensions"), pvecResult)))
	{
		pvecResult->clear();
		pvecResult->push_back( ".cr2" );
		pvecResult->push_back( ".crw" );
		pvecResult->push_back( ".dc2" );
		pvecResult->push_back( ".dcr" );
		pvecResult->push_back( ".dcs" );
		pvecResult->push_back( ".dng" );
		pvecResult->push_back( ".kdc" );
		pvecResult->push_back( ".mdc" );
		pvecResult->push_back( ".mrw" );
		pvecResult->push_back( ".nef" );
		pvecResult->push_back( ".orf" );
	}
}

void NJP2RawConfig::RegGetRawProfiles(bool bGetDefaultValue, NJR_ProfileMap &mapProfiles)
{
	mapProfiles.clear();

	bool bLoadedAny = false;
	bool bLoadedDefault = false;

	std::vector< std::string > vecProfileNames;

	NJR_RawProfile tempProf;

	if (!bGetDefaultValue
	&&	NLeoHelpers::LeetRegEnumKey(HKEY_CURRENT_USER, JP2RAW_REG_PATH, "Cfg", &vecProfileNames))
	{
		for (std::vector< std::string >::iterator pProfileName = vecProfileNames.begin(); pProfileName != vecProfileNames.end(); ++pProfileName)
		{
			const char *szProfileName = pProfileName->c_str();
			if ('_' == szProfileName[0]) { szProfileName++; } // Skip the separator.

			RegGetRawProfile(bGetDefaultValue, tempProf, szProfileName);
			mapProfiles.insert(std::pair< std::string, NJR_RawProfile >(tempProf.strSafeName, tempProf));

			if ('\0' == szProfileName[0]) { bLoadedDefault = true; }
			bLoadedAny = true;
		}
	}

	if (!bLoadedDefault)
	{
		RegGetRawProfile(bGetDefaultValue, tempProf, "");
		mapProfiles.insert(std::pair< std::string, NJR_RawProfile >(tempProf.strSafeName, tempProf));
	}

	if (!bLoadedAny)
	{
		// Default for Kodak camera that saves completely black thumbnails.
		RegGetRawProfile(true, tempProf, "");
		tempProf.strDisplayName = "Kodak / DCS Pro 14N";
		tempProf.strSafeName = "Kodak___DCS_Pro_14N";
		tempProf.rsThumbs.bTryPreview = false;
		tempProf.rsViewers.bTryPreview = false;
		mapProfiles.insert(std::pair< std::string, NJR_RawProfile >(tempProf.strSafeName, tempProf));
	}
}

void NJP2RawConfig::RegGetRawProfile(bool bGetDefaultValue, NJR_RawProfile &rawProfile, const char *szSafeName)
{
	const char *szDefaultProfileName = JP2RAW_REG_PATH "\\Cfg";
	std::string strProfileName = szDefaultProfileName;
	if ('\0' != szSafeName[0])
	{
		strProfileName += "_";
		strProfileName += szSafeName;
	}
	bool bLoadDefaultProfile = false;

	if (!bGetDefaultValue)
	{
		if (NLeoHelpers::LeetRegQueryStringValue(HKEY_CURRENT_USER, strProfileName.c_str(), "DisplayName", &rawProfile.strDisplayName))
		{
			rawProfile.strSafeName = szSafeName;
		}
		else
		{
			if ('\0' != szSafeName[0]
			&&	NLeoHelpers::LeetRegQueryStringValue(HKEY_CURRENT_USER, szDefaultProfileName, "DisplayName", &rawProfile.strDisplayName))
			{
				rawProfile.strSafeName = "";
				strProfileName = szDefaultProfileName;
			}
			else
			{
				bGetDefaultValue = true;
			}
		}
	}

	if (bGetDefaultValue)
	{
		rawProfile.strDisplayName = "< Default >";
		rawProfile.strSafeName = "";
	}

	RegGetRawSettings(bGetDefaultValue, strProfileName + "\\Thumbs",  rawProfile.rsThumbs,  true);
	RegGetRawSettings(bGetDefaultValue, strProfileName + "\\Viewers", rawProfile.rsViewers, false);
}

void NJP2RawConfig::RegGetRawSettings(bool bGetDefaultValue, const std::string &strSettingsPath, NJR_RawSettings &rs, bool bThumbs)
{
	DWORD dwType = REG_NONE;
	void *pData = NULL;
	DWORD dwDataSize = 0;

	NJR_RawSettings defaultRawSettings;
	// We mainly rely on the default set up by the NJR_RawSettings constructor.
	rs = defaultRawSettings;
	rs.bTryFull = (!bThumbs);

	if (!bGetDefaultValue)
	{
		// Overwrite the defaults with any values found in the registry.

		const char *p = strSettingsPath.c_str();

		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "TryPreview",     &rs.bTryPreview);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "TryFull",        &rs.bTryFull);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "Rotation",       &rs.iRotation);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "FujiTurn",       &rs.bFujiTurn);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "InterpQuality",  &rs.iInterpQuality);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "DocModeNoCol",   &rs.bDocModeNoCol);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "DocModeRaw",     &rs.bDocModeRaw);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "HalfSizeColor",  &rs.bHalfSizeColor);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "RGGB",           &rs.bRGGB);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "UseSecondary",   &rs.bUseSecondary);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "Bilateral",      &rs.bBilateral);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "BilatDomain",    &rs.iBilatDomain);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "BilatRange",     &rs.iBilatRange);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "AutoWhite",      &rs.bAutoWhite);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "CamWhite",       &rs.bCamWhite);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "SetWhite",       &rs.bSetWhite);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "UserMul1",       &rs.iUserMul1);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "UserMul2",       &rs.iUserMul2);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "UserMul3",       &rs.iUserMul3);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "UserMul4",       &rs.iUserMul4);
		NLeoHelpers::LeetRegQueryBoolValue(  HKEY_CURRENT_USER, p, "SetBlackPoint",  &rs.bSetBlackPoint);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "BlackPoint",     &rs.iBlackPoint);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "Brightness",     &rs.iBrightness);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "HighlightMode",  &rs.iHighlightMode);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "OutIccType",     &rs.iOutIccType);
		NLeoHelpers::LeetRegQueryStringValue(HKEY_CURRENT_USER, p, "OutIcc",         &rs.strOutIcc);
		NLeoHelpers::LeetRegQueryIntValue(   HKEY_CURRENT_USER, p, "CamIccType",     &rs.iCamIccType);
		NLeoHelpers::LeetRegQueryStringValue(HKEY_CURRENT_USER, p, "CamIcc",         &rs.strCamIcc);
	}
}

void NJP2RawConfig::RegSaveRawProfiles(NJR_ProfileMap &mapProfiles)
{
	// First, find and delete any existing profiles in the registry.

	std::vector< std::string > vecOldProfileNames;
	std::string strProfileName;

	if (NLeoHelpers::LeetRegEnumKey(HKEY_CURRENT_USER, JP2RAW_REG_PATH, "Cfg", &vecOldProfileNames))
	{
		for (std::vector< std::string >::iterator pProfileName = vecOldProfileNames.begin(); pProfileName != vecOldProfileNames.end(); ++pProfileName)
		{
			strProfileName = "Cfg";
			strProfileName += *pProfileName;

			NLeoHelpers::LeetRegDeleteKey(HKEY_CURRENT_USER, JP2RAW_REG_PATH, strProfileName.c_str());
		}
	}

	// Now save out all the profiles that are in memory.

	for (NJR_ProfileMap::iterator pPair = mapProfiles.begin(); pPair != mapProfiles.end(); ++pPair)
	{
		strProfileName = JP2RAW_REG_PATH "\\Cfg";
		if (!pPair->first.empty())
		{
			strProfileName += "_";
			strProfileName += pPair->first; // SafeName
		}

		NJR_RawSettings &rsThumbs  = pPair->second.rsThumbs;
		NJR_RawSettings &rsViewers = pPair->second.rsViewers;

		NLeoHelpers::LeetRegSetStringValue(HKEY_CURRENT_USER, strProfileName.c_str(), "DisplayName", pPair->second.strDisplayName.c_str());
		RegSaveRawSettings(strProfileName + "\\Thumbs",  rsThumbs);
		RegSaveRawSettings(strProfileName + "\\Viewers", rsViewers);
	}
}

void NJP2RawConfig::RegSaveRawSettings(const std::string &settingsPath, const NJR_RawSettings &rs)
{
	const char *p = settingsPath.c_str();

	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "TryPreview",     rs.bTryPreview);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "TryFull",        rs.bTryFull);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "Rotation",       rs.iRotation);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "FujiTurn",       rs.bFujiTurn);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "InterpQuality",  rs.iInterpQuality);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "DocModeNoCol",   rs.bDocModeNoCol);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "DocModeRaw",     rs.bDocModeRaw);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "HalfSizeColor",  rs.bHalfSizeColor);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "RGGB",           rs.bRGGB);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "UseSecondary",   rs.bUseSecondary);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "Bilateral",      rs.bBilateral);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "BilatDomain",    rs.iBilatDomain);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "BilatRange",     rs.iBilatRange);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "AutoWhite",      rs.bAutoWhite);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "CamWhite",       rs.bCamWhite);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "SetWhite",       rs.bSetWhite);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "UserMul1",       rs.iUserMul1);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "UserMul2",       rs.iUserMul2);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "UserMul3",       rs.iUserMul3);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "UserMul4",       rs.iUserMul4);
	NLeoHelpers::LeetRegSetBoolValue(  HKEY_CURRENT_USER, p, "SetBlackPoint",  rs.bSetBlackPoint);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "BlackPoint",     rs.iBlackPoint);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "Brightness",     rs.iBrightness);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "HighlightMode",  rs.iHighlightMode);
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "OutIccType",     rs.iOutIccType);
	NLeoHelpers::LeetRegSetStringValue(HKEY_CURRENT_USER, p, "OutIcc",         rs.strOutIcc.c_str());
	NLeoHelpers::LeetRegSetIntValue(   HKEY_CURRENT_USER, p, "CamIccType",     rs.iCamIccType);
	NLeoHelpers::LeetRegSetStringValue(HKEY_CURRENT_USER, p, "CamIcc",         rs.strCamIcc.c_str());
}

bool NJP2RawConfig::CheckOpus(DWORD64 dw64OpusVersion, NJR_Ability ability)
{
	switch(ability)
	{
	case(NTTA_RUN):
		return(dw64OpusVersion >= ((8i64<<48) | (2i64<<32) | (0i64<<16) | (0i64<<00)));
	default:
		return(false);
	}
}
